public class Account{

    /* The list of attributes, 
    defining the state of the object.*/
    private String accountHoldersName;
    private String accountNumber;
    private double amount;
    private String history;
    private static final long BANK_ID = 80056322;
    private static int numberOfAccounts = 0;

    /* This is a constructor. It desribes how to create a 
    new Account object*/
    public Account(String accountHoldersName, 
        String accountNumber, 
        double amount){
        this.accountHoldersName = accountHoldersName;
        this.accountNumber = accountNumber;
        this.amount = amount;
        history ="";
        numberOfAccounts++; 
        // numberOfAccount = numberOfAccount + 1;
    }


    /* a class can have more than one constructor */
    public Account(String accountHoldersName, 
        String accountNumber){
        this.accountNumber = accountNumber;
        this.accountHoldersName = accountHoldersName;
        /*We do not need 'this.variable here', because there
          is no way to confuse these variables with something else.*/
        amount = 0.0;
        history = "";
        numberOfAccounts++;
    }

    /* This will not work, because the constructor is not unique */
    /*public Account(String accountHoldersName, 
        String history){
        accountNumber = generateNewAccountNumber();
        this.accountHoldersName = accountHoldersName;
        amount = 0.0;
        this.history = "";
        numberOfAccounts++;
    }*/


    /* If we still want the standard constructor, 
    we have to explicitly define it*/
    // public Account () { }

    /* The list of methods forming the behavior of the
    account class*/

    public void processTransfer(double amount){
        if(!isAmountPositive() && amount < 0){
            updateHistory("Transaction could not be completed.");
        }else{
            this.amount = this.amount + amount;
            updateHistory("Amount: "+amount+" was processed on your"
                +" account. New balance is: "+this.amount);
        }    	
    }

    private void updateHistory(String entry){
        history = history + "\n" + entry;
    }

    public boolean isAmountPositive(){
    	return amount >= 0;
    }

    /* This method defines how the textual representation of
    your object will look like. It is reused e.g. by System.out.println()
    to print Account objects.*/
    public String toString(){
        return "Bank Id: " + BANK_ID 
        + "\nName: "+ accountHoldersName +
        "\nAccount Number: "+ accountNumber + "\nAmount: " + amount;
    }

    /* Getter Methods*/
    public String getAccountHoldersName(){
        return accountHoldersName;
    }

    public static long getBankId(){
        return BANK_ID;
    }

    public String getAccountNumber(){
        return accountNumber;
    }

    public double getAmount(){
        return amount;
    }

    public String getHistory(){
        return history;
    }

    public static int getNumberOfAccounts(){
        /*updateHistory("The number of accounts has
            been requested.");*/
        return numberOfAccounts;
    }
 
}