import java.io.*;

public class Assignment2{

	static final String FILENAME_USERS = "users.csv";
	static final String FILENAME_INFO = "info.csv";

	public static void main(String[] args) throws Exception {
	
		String input = askForString("Welcome. Are you a new user? (y/n) ");

		if(input.equals("y")){
			
			registerNewUser();

		}else if (input.equals("n")){
			
			String email = askForString("Please enter your email address: ");
			String password = askForString("Please enter your password: ");
			
			if (!checkUserCredentials(FILENAME_USERS, email, password)){
				System.out.println("Username or password did not match.");
				return;
			}

			System.out.println("You are logged in.");

			outputUserInfo(FILENAME_INFO, email);

		}else {

			System.out.println("This is not a valid input.");
			return;
		}
	}

	static void registerNewUser() throws IOException{
		
		String firstName = askForString("Please enter your first name: ");
		if(!validateName(firstName)){
			System.out.println("This is not a valid first name.");
			return;
		}
		String lastName = askForString("Please enter your last name: ");
		if(!validateName(lastName)){
			System.out.println("This is not a valid last name.");
			return;				
		}

		String email = askForString("Please enter your email address: ");
		if(!validateEmail(FILENAME_USERS, email)){
			System.out.println("This email has been registered already.");
			return;
		}

		String password1 = askForString("Please enter your password: ");
		String password2 = askForString("Please confirm your password: ");
		if(!validatePassword(password1, password2)){
			System.out.println("The passwords you entered do not match.");
			return;
		}

		createNewUser(firstName, lastName, email, password1);

		String twitter = askForString("Please enter your Twitter handle: ");
		String github = askForString("Please enter your Github username: ");
		String website = askForString("Please enter your website address: ");
		String university = askForString("Please enter the name of your university: ");
		String major = askForString("Please enter your major: ");
		String company = askForString("Please enter your company name: ");
		String title = askForString("Please enter your title: ");
		int number = askForInt("Please enter your phone number (10 digits): ");

		storeUserInfo(email, twitter, github, website, university, major, company, title, number);
	}

	static boolean validateName(String name){
		return name.matches("[a-zA-z]*");
	}

	static boolean validateEmail(String fileName, String newEmail) throws IOException{
		
		FileInputStream fis = new FileInputStream(new File(fileName));
 		
 		String record;
 		String email;
 		char character;
		int i  = fis.read();

		// read in whole file
		while (i != -1){

			record = "";
			character = (char)i;

			// read in one record
			while (character != ';') {
				record = record + character;
				character = (char)fis.read();					
			}
			
			email = record.substring(0,record.indexOf(','));
			
			//System.out.println("Comparing "+email +" with " + newEmail);

			if(email.equals(newEmail)){ // email found in record
				//System.out.println("Found match: " + email);
				fis.close();
				return false;				
			}
			i = fis.read();		
		}

		fis.close(); // email was not found in record
		return true;
	}

	static boolean validatePassword(String password1, String password2){
		return password1.equals(password2);
	}

	static String askForString(String message) throws IOException{
		System.out.println();
		System.out.print(message);

		BufferedReader input = new BufferedReader(new InputStreamReader(System.in));
		
		return input.readLine();
	}

	static int askForInt(String message) throws IOException{
		System.out.println();
		System.out.print(message);

		BufferedReader input = new BufferedReader(new InputStreamReader(System.in));
		
		return Integer.parseInt(input.readLine());
	}

	static boolean checkUserCredentials(String fileName, String email, String password) throws IOException{
			
		FileInputStream fis = new FileInputStream(new File(fileName));
 		
 		String record;
 		String currentEmail;
 		String currentPassword;
 		char character;
		int i  = fis.read();

		// read in whole file
		while (i != -1){

			record = "";
			character = (char)i;

			// read in one record
			while (character != ';') {
				record = record + character;
				character = (char)fis.read();					
			}
			currentEmail = record.substring(0,record.indexOf(','));
			currentPassword = record.substring(record.indexOf(',')+1,record.indexOf(',',record.indexOf(',')+1));
			
			//System.out.println("current email: "+ currentEmail +",current password: " + currentPassword);
			if(currentEmail.equals(email)){
				fis.close();
				if(currentPassword.equals(password)){
					//System.out.println("Found match: " + password +","+currentPassword);
					return true;
				}else{
					//System.out.println("Password for user does not match: " + email);
					return false;
				}
							
			}
			i = fis.read();		
		}

		// finished parsing the file
		fis.close();
		return false;
	}
	static void createNewUser(String firstName, String lastName, String email, String password1) throws FileNotFoundException{

		String record = email+","+password1+","+firstName+","+lastName+";"; //\r\n
		append(FILENAME_USERS, record);
	}

	static void storeUserInfo(String email, String twitter, String github, String website, String university, String major, String company, String title, int number) throws FileNotFoundException{

		String record = email+","+twitter+","+github+","+website+","+university+","+major+","+company+","+title+","+number+";";
		append(FILENAME_INFO, record);
	}


	static void outputUserInfo(String fileName, String email) throws IOException{

		String record;
 		char character;
 		String currentEmail;

		String twitter;
		String github;
		String website;
		String university;
		String major;
		String company;
		String title;
		String number;

		FileInputStream fis = new FileInputStream(new File(fileName));
		int i  = fis.read();

		while (i != -1){

			record = "";
			character = (char)i;

			// read in one record
			while (character != ';') {
				record = record + character;
				character = (char)fis.read();					
			}

			currentEmail = record.substring(0,record.indexOf(','));
			
			if(currentEmail.equals(email)){
				fis.close();
				number = record.substring(record.lastIndexOf(',')+1);
				record = record.substring(0,record.lastIndexOf(','));
				title = record.substring(record.lastIndexOf(',')+1);
				record = record.substring(0,record.lastIndexOf(','));
				company = record.substring(record.lastIndexOf(',')+1);
				record = record.substring(0,record.lastIndexOf(','));
				major = record.substring(record.lastIndexOf(',')+1);
				record = record.substring(0,record.lastIndexOf(','));
				university = record.substring(record.lastIndexOf(',')+1);
				record = record.substring(0,record.lastIndexOf(','));
				website = record.substring(record.lastIndexOf(',')+1);
				record = record.substring(0,record.lastIndexOf(','));
				github = record.substring(record.lastIndexOf(',')+1);
				record = record.substring(0,record.lastIndexOf(','));
				twitter = record.substring(record.lastIndexOf(',')+1);

				System.out.println("Twitter: "+ twitter);
				System.out.println("Github: "+ github);
				System.out.println("Website: "+ website);
				System.out.println("University: "+ university);
				System.out.println("Major: "+ major);
				System.out.println("Company: "+ company);				
				System.out.println("Title: "+ title);
				System.out.println("Telephone Number: "+ number);
				return;
										
			}
			i = fis.read();		
		}
		System.out.println("Error: Could not find user data.");
		fis.close();
	}

	static void append(String fileName, String record) throws FileNotFoundException{
		PrintWriter out = new PrintWriter(new FileOutputStream(new File(fileName), true));
		out.append(record);
		out.close();
	}


}