import java.io.*;
import java.util.Scanner;

public class ZerplyAssignment {
	public static void main(String[] args) throws Exception {
		userMenu();
	}

	//this method welcomes the user takes the initial input
	static void userMenu() throws Exception {
		final String FILENAME_USERS = "users.csv";
		final String FILENAME_INFO = "info.csv";

		String [] all_records = readInputFile(FILENAME_USERS);

		String input = askForString("Welcome. Are you a new user? (y/n) ");

		if(input.equals("y")){
			String firstName = askForString("Please enter your first name: ");
			if(!validateAlpha(firstName)){
				System.out.println("This is not a valid first name.");
			}
			String lastName = askForString("Please enter your last name: ");
			if(!validateAlpha(lastName)){
				System.out.println("This is not a valid last name.");
			}

			String email = askForString("Please enter your email address: ");
			if (!validateEmail(email)) {
				System.out.println("That is not a valid email address.");
				return;
			}
			if(checkExistingUser(email, all_records)){
				System.out.println("This email has been registered already.");
				return;
			}
			else {
				String password1 = askForString("Please enter your password: ");
				String password2 = askForString("Please confirm your password: ");
				if(!validatePassword(password1, password2)){
					System.out.println("The passwords you entered do not match.");
				}
				String newUser = storeUser(email,password1,firstName, lastName);
				outputInfo(newUser,FILENAME_USERS);
			}

			String new_user_record = storeUserInfo();
			if (new_user_record.equals("invalid")) {
				System.out.println("Your record was not written because of invalid input.");
			}
			else {
				new_user_record = email+","+new_user_record;	//append the email in order to locate the information later
				outputInfo(new_user_record, FILENAME_INFO);
				System.out.println();
				System.out.println("Thank you for registering!");
			}

		}else if (input.equals("n")){
			
			String email = askForString("Please enter your email address: ");
			String password = askForString("Please enter your password: ");
			
			if (!userLogIn(email, password, all_records)){
				System.out.println("Username or password did not match.");
			}
			else {
				System.out.println();
				System.out.println("You are logged in.");
				System.out.println();
				printUserProfile(FILENAME_INFO, email);
			}

		}else {
			System.out.println("This is not a valid input.");
		}
	}

	//this method reads existing data from user information files
	static String [] readInputFile (String filename) throws IOException{ 
		File file = new File(filename);
		Scanner csvScanner = new Scanner(file);
		String record = "";
		String [] all_records = new String [5];
		
		while (csvScanner.hasNextLine()){
			record = csvScanner.nextLine();
			all_records = record.split(";");
		}

		return all_records;
	}

	//this method checks to see if the user logging in exists in the input file already
	static boolean checkExistingUser (String email, String [] userInfo) { 
		String [] new_record = new String [10];
		boolean userExists = false;
		if (userInfo[0]==null){
			userExists = false;
		}
		else {
			for (int i=0;i<userInfo.length;i++) {
				new_record = userInfo[i].split(",");
				if (new_record[0].equals(email)){
						userExists = true;
				}
			}
		}

		return userExists;
	}

	//this method takes user input for new users and stores it in a comma separated String
	static String storeUser (String email, String password, String firstName,  String lastName) { 
		return email+","+password+","+firstName+","+lastName+";";
	}

	//this method writes a String with user info to the users.csv or info.csv (make sure you don’t overwrite your info!)
	static void outputInfo (String record, String fileName) throws FileNotFoundException{
		PrintWriter out = new PrintWriter(new FileOutputStream(new File(fileName), true));
		out.append(record);
		out.close();
	}

	//this method takes user provided email and password and checks it against the data from the user.csv file
	static boolean userLogIn (String email, String password, String [] userInfo) { 
		String [] new_record = new String [10];
		boolean passMatch = false;
		if (userInfo[0]==null){
			passMatch = false;
		}
		else {
			for (int i=0;i<userInfo.length;i++) {
				new_record = userInfo[i].split(",");
				if (new_record[0].equals(email)){
					if(new_record[1].equals(password)) {
						passMatch = true;
					}
				}
			}
		}
		
		return passMatch;
	}

	//this method takes a user provided email and the info.csv file object, pulls the profile info of the user with the provided email
	static void printUserProfile (String fileName, String email) throws IOException{
		String [] all_records = new String [9];
		String [] new_record = new String [5];
		
		all_records = readInputFile(fileName);
		
		if (all_records[0]==null){
			return;
		}
		else {
			for (int i=0;i<all_records.length;i++) {
				new_record = all_records[i].split(",");
				if (new_record[0].equals(email)){
					System.out.println("Email: " + new_record[0]);		//email
					System.out.println("Twitter: " + new_record[1]);	//twitter
					System.out.println("GitHub: " + new_record[2]);		//github
					System.out.println("Website: " + new_record[3]);	//website
					System.out.println("University: " + new_record[4]);	//university
					System.out.println("Major: " + new_record[5]);		//major
					System.out.println("Company: " + new_record[6]);	//company
					System.out.println("Title: " + new_record[7]);		//title
					System.out.println("Phone: " + new_record[8]);		//phone
				}
			}
		}

	}

	//this method asks the user for profile info and then stores it in a comma separated String
	static String storeUserInfo () throws IOException {
		boolean validProfile = true;

		String twitter = askForString("Please enter your Twitter handle: ");
		if (!validateAlphaNum(twitter)) { System.out.println("That is not a valid Twitter handle"); validProfile = false; }
		String github = askForString("Please enter your Github username: ");
		if (!validateAlphaNum(github)) { System.out.println("That is not a valid GitHub name"); validProfile = false; }
		String website = askForString("Please enter your website address: ");
		if (!validateWebsite(website)) { System.out.println("That is not a valid website"); validProfile = false; }
		String university = askForString("Please enter the name of your university: ");
		if (!validateAlphaSpace(university)) { System.out.println("That is not a valid university name"); validProfile = false; }
		String major = askForString("Please enter your major: ");
		if (!validateAlphaSpace(major)) { System.out.println("That is not a valid major"); validProfile = false; }
		String company = askForString("Please enter your company name: ");
		if (!validateAlphaNumSpace(company)) { System.out.println("That is not a valid company name"); validProfile = false; }
		String title = askForString("Please enter your title: ");
		if (!validateAlphaNumSpace(title)) { System.out.println("That is not a valid title"); validProfile = false; }
		String number = askForString("Please enter your phone number (10 digits): ");
		if (!validatePhoneNumber(number)) { System.out.println("That is not a valid phone number"); validProfile = false; }

		if (validProfile==true) {
			String new_user_record = twitter+","+github+","+website+","+university+","+major+","+company+","+title+","+number+";";
			return new_user_record;
		}
		else {
			return "invalid";
		}
	}

	// *** HELPER METHODS ***

	static boolean validateAlpha(String name) {
		return name.matches("[a-zA-Z]*");
	}

	static boolean validateAlphaSpace(String name) {
		return name.matches("[a-zA-Z\\s]*");
	}

	static boolean validateAlphaNum(String input) {
		return input.matches("[a-zA-Z0-9]*");
	}

	static boolean validateAlphaNumSpace(String input) {
		return input.matches("[a-zA-Z0-9\\s]*");
	}

	static boolean validateEmail (String email) {
		return email.matches("[\\w\\.][\\w]*@[\\w]\\.[a-z]{2,3}");
	}

	//this method checks if the two user entered emails are the same
	static boolean validatePassword (String password1, String password2) { 
		return password1.equals(password2); 
	}

	static boolean validateWebsite(String input) {
		return input.matches("^www\\.[a-z0-9]*\\.[a-z]{2,3}");
	}

	static boolean validatePhoneNumber(String input) {
		return input.matches("[0-9]{3}-[0-9]{3}-[0-9]{4}");
	}

	static String askForString (String message) throws IOException{
		System.out.println();
		System.out.print(message);

		BufferedReader input = new BufferedReader(new InputStreamReader(System.in));
		
		return input.readLine();
	}

}