import mest.entity.*;
import mest.utility.*;

import java.net.URL;
import java.net.MalformedURLException;

import java.util.HashSet;
import java.util.ArrayList;

import java.util.GregorianCalendar;

import org.junit.Test;
import static org.junit.Assert.*;

public class AssignmentLab{

	@Test
	public void testAssignment() {

		URL url;
		GregorianCalendar startDate;
		GregorianCalendar endDate;

		//this should cause a compiler error
		//CareerStage s = new CareerStage("1",null,null,nul,"",null);

		// Testing Theme Managing
		try{
			
			url = new URL("http://www.mestly.com/css/style.css");

			Theme butterflyTheme = new Theme("The Butterfly Effect", url);
			Theme conservativeTheme = new Theme("Conservative", url);
			Theme fancyTheme = new PremiumTheme("Old Spice", url, 25.d);
			Theme businessTheme = new PremiumTheme("Serious Business", url, 15.d);

			ThemeManager.addTheme(butterflyTheme);
			ThemeManager.addTheme(conservativeTheme);
			ThemeManager.addTheme(fancyTheme);
			ThemeManager.addTheme(businessTheme);

			assertTrue("Add theme",ThemeManager.getThemeList().size()==4);
			assertTrue("Add theme uid generation",ThemeManager.getThemeList().get(2).getUid()==2);
			assertTrue("Add theme uid generation",ThemeManager.getThemeList().get(0).getUid()==0);

			ThemeManager.removeTheme(fancyTheme.getUid());

			assertTrue("Remove theme",ThemeManager.getThemeList().size()==3);

			assertNotNull("Default theme in theme manager", ThemeManager.getDefaultTheme());

		}catch(MalformedURLException exception){

			assertTrue("Error in Test Suite",false);			

		}

		// Testing User Managing

		User falk = new User("Falk", "Benke", "fbenke", "falk@meltwater.org", "holiday", 
			"Teaching Fellow at MEST");
		User rowdy = new User("Ben", "Sholes", "bsholes", "rowdy@meltwater.org", "ubuntu", 
			"Tech Fellow at MEST");
		User nikunj = new User("Nikunj", "Handa", "nhanda", "nknj@meltwater.org", "django", 
			"Web Evangelist at MEST");
		User brooke = new User("Falk", "Cassooff", "bcassoff", "brooke@meltwater.org", "buzz", 
			"Comm Fellow at MEST");
		User thea = new User("Theandra", "Sokolkowski", "tsoko", "thea@meltwater.org", "branding", 
			"Branding Expert at MEST");
		User colin = new User("Colin", "Casey", "ccasey", "colin@meltwater.org", "financial modeling", 
			"Financial Specialist at MEST");
		User anca = new User("Anca", "Wade", "awade", "anca@meltwater.org", "business", 
			"Business Fellow at MEST");

		assertNotNull("Default theme set in user", falk.getActiveTheme());
		falk.setActiveTheme(ThemeManager.getThemeList().get(0));
		assertTrue("Accessing theme",falk.getActiveTheme().getTitle().equals("The Butterfly Effect"));

		UserManager.addUser(falk);
		UserManager.addUser(rowdy);
		UserManager.addUser(nikunj);
		UserManager.addUser(thea);
		UserManager.addUser(colin);
		UserManager.addUser(brooke);
		UserManager.addUser(anca);

		User fakeUser = new User("Scooby", "Doo", "fbenke", "scooby@mysterycompany.com", "shaggy", 
			"Private Investigator at Mystery Company");

		assertFalse("Adding user with duplicate username",UserManager.addUser(fakeUser));
		assertTrue("Adding users to manager",UserManager.getUserList().size() == 7);

		// Testing Tag List in User
		falk.addTag("HPI");
		falk.addTag("Machine Learning");
		falk.addTag("Theoretical Computer Science");
		falk.addTag("Schuhplattler");

		assertTrue("Adding Tags",falk.getTagList().size() == 4);

		falk.addTag("Machine Learning");

		assertTrue("No duplicates",falk.getTagList().size() == 4);

		assertFalse("Remove Tag",falk.removeTag("IOS"));

		assertTrue("Tags in adding order", falk.getTagList().get(1).equals("Machine Learning") && 
		falk.getTagList().get(2).equals("Theoretical Computer Science") &&			
		falk.getTagList().get(0).equals("HPI") && falk.getTagList().get(3).equals("Schuhplattler") );

		assertTrue("Remove Tag Failure",falk.removeTag("Schuhplattler"));

		// Testing Service Manager
		ServiceManager.addSupportedService("LinkedIn");
		ServiceManager.addSupportedService("GitHub");
		ServiceManager.addSupportedService("Twitter");
		ServiceManager.addSupportedService("Twitter");
		ServiceManager.addSupportedService("Facebook");
		ServiceManager.addSupportedService("PersonalWebsite");

		assertTrue("Add Supported Services",
			ServiceManager.getSupportedServices().size() == 5);

		assertTrue("Remove Supported Service", ServiceManager.removeSupportedService("Facebook"));

		assertFalse("Remove Supported Service repeatedly",ServiceManager.removeSupportedService("Facebook"));

		assertTrue("Positive check for supported service", ServiceManager.isSupportedService("PersonalWebsite"));

		assertFalse("Negative check for supported service", ServiceManager.isSupportedService("Vimeo"));

		// Testing Service List in User
		try{
			
			url = new URL("http://www.dummyaddress.com/fbenke");
			assertTrue("Add supported service for user",falk.addService("LinkedIn",url));
			
			falk.addService("Twitter", url);
			falk.addService("PersonalWebsite",url);
			assertFalse("Add unsupported service for user",falk.addService("Facebook", url));
			assertFalse("Add duplicate service for user",falk.addService("Twitter", url));
			
			assertTrue("Count supported services for user",falk.getServiceList().size()==3);

			assertTrue("Remove service for user",falk.removeService("Twitter"));
			assertFalse("Remove service for user",falk.removeService("Twitter"));

			assertTrue("Count supported services for user after removing",falk.getServiceList().size()==2);
		}catch(MalformedURLException exception){

			assertTrue("Error in Test Suite",false);			

		}

		// Testing Following List in User

		falk.addFollowing(rowdy.getUsername());
		falk.addFollowing(nikunj.getUsername());
		falk.addFollowing(brooke.getUsername());
		falk.addFollowing(thea.getUsername());
		falk.addFollowing(anca.getUsername());
		falk.addFollowing(anca.getUsername());
		assertTrue("Add usernames to following list",falk.getFollowingList().size() == 5);

		assertTrue("Remove username following list", falk.removeFollowing(rowdy.getUsername()));
		assertFalse("Remove username from following list ", falk.removeFollowing(colin.getUsername()));

		anca.addFollowing(falk.getUsername());
		rowdy.addFollowing(falk.getUsername());
		nikunj.addFollowing(falk.getUsername());
		thea.addFollowing(falk.getUsername());

		assertTrue("Get list of followers", (UserManager.getFollowers(falk.getUsername())).size()==4);

		assertTrue("Removing user from manager (success)", UserManager.removeUser("ccasey"));
		assertFalse("Removing user from manager (failure)", UserManager.removeUser("rtanksley"));

		assertTrue("Adding users to manager",UserManager.getUserList().size() == 6);

		// Testing University and Work List
		try{
			url = new URL("http://www.dummyaddress.com/fbenke");
	
			startDate = new GregorianCalendar(2006,9,1); 	
			endDate = new GregorianCalendar(2009,8,30); 	
			University undergrad = new University("Berlin School of Economics and Law", "Bachelor of Science",
				"Business Information Systems", startDate, endDate, url, "Berlin", url);
			Work ibm = new Work("IBM", "BA Student", "6 internships in various departments",
			startDate, endDate, url, "Berlin", url);
			startDate = new GregorianCalendar(2010,3,1); 	
			endDate = new GregorianCalendar(2013,8,30); 
			University grad = new University("Hasso Plattner Institute", "Master of Science",
				"IT Systems Engineering", startDate, endDate, url, "Potsdam", url);
			startDate = new GregorianCalendar(2009,9,1); 	
			endDate = new GregorianCalendar(2010,2,31); 
			Work dealstreet = new Work("Upside Shopping GmbH", "Internship Q&A", 
				"Q&A, Customer Support, Product Management", startDate, endDate, url, "Berlin", url);
			startDate = new GregorianCalendar(2010,0,1); 	
			endDate = new GregorianCalendar(2010,2,31); 
			Work project = new Work("University of Potsdam", "Tutor Theoretical Computer Science",
				"Tutoring Students, Grading Homework", startDate, endDate, url, "Potsdam", url);
			startDate = new GregorianCalendar(2013,7,1); 	
			endDate = new GregorianCalendar(2014,5,30); 
			Work mest = new Work("Meltwater Entrepreneurial School of Technology", "Teaching Fellow", 
				"Teaching and Mentoring Students, Grading, Organizing Capstone Projects", startDate,
				endDate, url, "Accra", url);

			falk.addUniversity(grad);
			assertTrue("Adding university attendance", falk.addUniversity(undergrad));

			falk.addWork(mest);
			falk.addWork(dealstreet);
			assertTrue("Adding work", falk.addWork(ibm));
			falk.addWork(project);

			/*for(University u: falk.getUniversityList()){
				System.out.println(u.getName()+","+u.getUid());
			}
			for(Work w: falk.getWorkList()){
				System.out.println(w.getName()+","+w.getUid());
			}*/

			assertTrue("Sorting universities", falk.getUniversityList().get(0).getName().equals("Berlin School of Economics and Law")
				&& falk.getUniversityList().get(1).getName().equals("Hasso Plattner Institute"));
			
			assertTrue("Sorting work", falk.getWorkList().get(0).getName().equals("IBM") &&
				falk.getWorkList().get(1).getName().equals("University of Potsdam") && 
				falk.getWorkList().get(2).getName().equals("Upside Shopping GmbH") &&
				falk.getWorkList().get(3).getName().equals("Meltwater Entrepreneurial School of Technology"));

			assertTrue("Unique ids for career stages", findDuplicates(falk.getUniversityList()) && 
				findDuplicates(falk.getWorkList()));
			
			falk.removeUniversity(falk.getUniversityList().get(0).getUid());
			assertTrue("Removing university", falk.getUniversityList().get(0).getName().equals("Hasso Plattner Institute"));

			falk.removeWork(falk.getWorkList().get(2).getUid());
			assertTrue("Removing work", falk.getWorkList().get(2).getName().equals("Meltwater Entrepreneurial School of Technology"));

			System.out.println("Great Job!!!");


		}catch(MalformedURLException exception){
			assertTrue("Error in Test Suite",false);			
		}
	}

	private static boolean findDuplicates(ArrayList<? extends CareerStage> list){
		HashSet<Long> uniqueList = new HashSet<Long>();
		int i = 0;
		for(CareerStage obj : list) {
			uniqueList.add(obj.getUid());
			i++;
		}

		return uniqueList.size() == list.size();
	}
}