import mest.entity.*;
import mest.utility.*;

import java.net.URL;
import java.net.MalformedURLException;

import java.util.HashSet;
import java.util.ArrayList;

import java.util.GregorianCalendar;

// JUnit annotations
import org.junit.Ignore;
import org.junit.Test;
import org.junit.BeforeClass;

// JUnit assers
import static org.junit.Assert.*;

/* 
* Important: test cases are not completely independent from each other.
* They can be executed in arbitrary order, but not in parallel.
*/
public class ModularAssignmentLab{

	@BeforeClass
	public static void initializeDataStructures() throws MalformedURLException{
		
		URL url = new URL("http://www.mestly.com/css/style.css");
		GregorianCalendar startDate;
		GregorianCalendar endDate;

		// initialize themes
		Theme butterflyTheme = new Theme("The Butterfly Effect", url);
		Theme conservativeTheme = new Theme("Conservative", url);
		Theme fancyTheme = new PremiumTheme("Old Spice", url, 25.d);
		Theme businessTheme = new PremiumTheme("Serious Business", url, 15.d);

		ThemeManager.addTheme(butterflyTheme);
		ThemeManager.addTheme(conservativeTheme);
		ThemeManager.addTheme(fancyTheme);
		ThemeManager.addTheme(businessTheme);

		// initialize supported services
		ServiceManager.addSupportedService("LinkedIn");
		ServiceManager.addSupportedService("GitHub");
		ServiceManager.addSupportedService("Twitter");
		ServiceManager.addSupportedService("Twitter");
		ServiceManager.addSupportedService("Facebook");
		ServiceManager.addSupportedService("PersonalWebsite");

		// initialize users
		User falk = new User("Falk", "Benke", "fbenke", "falk@meltwater.org", "holiday", 
			"Teaching Fellow at MEST");
		User rowdy = new User("Ben", "Sholes", "bsholes", "rowdy@meltwater.org", "ubuntu", 
			"Tech Fellow at MEST");
		User nikunj = new User("Nikunj", "Handa", "nhanda", "nknj@meltwater.org", "django", 
			"Web Evangelist at MEST");
		User brooke = new User("Falk", "Cassooff", "bcassoff", "brooke@meltwater.org", "buzz", 
			"Comm Fellow at MEST");
		User thea = new User("Theandra", "Sokolkowski", "tsoko", "thea@meltwater.org", "branding", 
			"Branding Expert at MEST");
		User colin = new User("Colin", "Casey", "ccasey", "colin@meltwater.org", "financial modeling", 
			"Financial Specialist at MEST");
		User anca = new User("Anca", "Wade", "awade", "anca@meltwater.org", "business", 
			"Business Fellow at MEST");

		falk.addTag("HPI");
		falk.addTag("Machine Learning");
		falk.addTag("Theoretical Computer Science");
		falk.addTag("Schuhplattler");

		url = new URL("http://www.service.com/useraccount");

		falk.addService("Twitter", url);
		falk.addService("PersonalWebsite",url);
	
		falk.addFollowing(rowdy.getUsername());
		falk.addFollowing(nikunj.getUsername());
		falk.addFollowing(brooke.getUsername());
		falk.addFollowing(thea.getUsername());
		falk.addFollowing(anca.getUsername());
		falk.addFollowing(anca.getUsername());

		url = new URL("http://www.dummyaddress.com/fbenke");

		startDate = new GregorianCalendar(2006,9,1); 	
		endDate = new GregorianCalendar(2009,8,30); 	
		University undergrad = new University("Berlin School of Economics and Law", "Bachelor of Science",
			"Business Information Systems", startDate, endDate, url, "Berlin", url);
		Work ibm = new Work("IBM", "BA Student", "6 internships in various departments",
		startDate, endDate, url, "Berlin", url);
		startDate = new GregorianCalendar(2010,3,1); 	
		endDate = new GregorianCalendar(2013,8,30); 
		University grad = new University("Hasso Plattner Institute", "Master of Science",
			"IT Systems Engineering", startDate, endDate, url, "Potsdam", url);
		startDate = new GregorianCalendar(2009,9,1); 	
		endDate = new GregorianCalendar(2010,2,31); 
		Work dealstreet = new Work("Upside Shopping GmbH", "Internship Q&A", 
			"Q&A, Customer Support, Product Management", startDate, endDate, url, "Berlin", url);
		startDate = new GregorianCalendar(2010,0,1); 	
		endDate = new GregorianCalendar(2010,2,31); 
		Work project = new Work("University of Potsdam", "Tutor Theoretical Computer Science",
			"Tutoring Students, Grading Homework", startDate, endDate, url, "Potsdam", url);
		startDate = new GregorianCalendar(2013,7,1); 	
		endDate = new GregorianCalendar(2014,5,30); 
		Work mest = new Work("Meltwater Entrepreneurial School of Technology", "Teaching Fellow", 
			"Teaching and Mentoring Students, Grading, Organizing Capstone Projects", startDate,
			endDate, url, "Accra", url);

		falk.addUniversity(grad);
		falk.addUniversity(undergrad);
		falk.addWork(mest);
		falk.addWork(dealstreet);
		falk.addWork(project);
		falk.addWork(ibm);

		anca.addFollowing(falk.getUsername());
		rowdy.addFollowing(falk.getUsername());
		nikunj.addFollowing(falk.getUsername());
		thea.addFollowing(falk.getUsername());
		
		UserManager.addUser(falk);
		UserManager.addUser(rowdy);
		UserManager.addUser(nikunj);
		UserManager.addUser(thea);
		UserManager.addUser(colin);
		UserManager.addUser(brooke);
		UserManager.addUser(anca);
	}

	@Test
	public void testThemeManagement() {
		assertTrue("Failure while adding theme",ThemeManager.getThemeList().size()==4);
		
		assertTrue("Failure while generating uid (2)",ThemeManager.getThemeList().get(2).getUid()==2);
		
		assertTrue("Failure while generating uid (0)",ThemeManager.getThemeList().get(0).getUid()==0);

		assertTrue("Failure while removing theme", ThemeManager.removeTheme(ThemeManager.getThemeList().get(2).getUid()));

		assertFalse("Failure while removing non-existent theme", ThemeManager.removeTheme(4));

		assertTrue("Failure while removing theme, wrong list size",ThemeManager.getThemeList().size()==3);

		assertNotNull("No default theme set in theme manager", ThemeManager.getDefaultTheme());
	}

	@Test
	public void testThemeInUser() {
		assertNotNull("No default theme set in user", UserManager.getUser("fbenke").getActiveTheme());
		UserManager.getUser("fbenke").setActiveTheme(ThemeManager.getTheme(0));
		assertTrue("Accessing theme",UserManager.getUser("fbenke").getActiveTheme().getTitle().equals("The Butterfly Effect"));
	}

	@Test
	public void testUniqueUsername() {
		User scooby = new User("Scooby", "Doo", "fbenke", "scooby@mysterycompany.com", "shaggy", 
			"Private Investigator at Mystery Company");
		assertFalse("Failure while adding user with duplicate username",UserManager.addUser(scooby));
		assertTrue("Failure while adding user with duplicate username, wrong list size",UserManager.getUserList().size() == 7);
	}

	@Test
	public void testAddingTags() {
		assertTrue("Failure while adding tags, wrong list size",UserManager.getUser("fbenke").getTagList().size() == 4);
	}

	@Test
	public void testAddingDuplicateTags(){
		assertFalse("Failure while adding duplicate tag",UserManager.getUser("fbenke").addTag("Machine Learning"));
		assertTrue("No duplicates",UserManager.getUser("fbenke").getTagList().size() == 4);		
	}

	@Test
	public void testRemovingNonExistentTags(){
		assertFalse("Failure while removing non-existent tag", UserManager.getUser("fbenke").removeTag("IOS"));
	}

	@Test
	public void testingTagOrder(){
		assertTrue("Tags are not in right order", 
			UserManager.getUser("fbenke").getTagList().get(0).equals("HPI") && 
			UserManager.getUser("fbenke").getTagList().get(1).equals("Machine Learning") && 
			UserManager.getUser("fbenke").getTagList().get(2).equals("Theoretical Computer Science") &&			
			UserManager.getUser("fbenke").getTagList().get(3).equals("Schuhplattler") );
	}

	@Test
	public void testRemovingTag(){
		assertTrue("Failure while removing non-existent tag",UserManager.getUser("fbenke").removeTag("Schuhplattler"));
		assertTrue("Failure while restoring tag list", UserManager.getUser("fbenke").addTag("Schuhplattler"));
	}

	@Test
	public void testSupportedServiceListSize(){
		assertTrue("Wrong list size for supported services",
			ServiceManager.getSupportedServices().size() == 5);
	}

	@Test
	public void testRemovingNonExistentService(){
		assertFalse("Failure while removing non-existent service",ServiceManager.removeSupportedService("Xing"));
	}

	@Test
	public void testRemovingSupportedService(){
		assertTrue("Failure while removing non-existent service", ServiceManager.removeSupportedService("Facebook"));
		assertTrue("Failure while restoring supported services list", ServiceManager.addSupportedService("Facebook"));
	}

	@Test
	public void testCheckingSupportedServices(){
		assertTrue("Failure while checking for supported service", ServiceManager.isSupportedService("PersonalWebsite"));
	}

	@Test
	public void testCheckingSupportedServicesNegative(){
		assertFalse("Failure while checking for supported service (negative case)", ServiceManager.isSupportedService("Vimeo"));
	}

	@Test
	public void testRemovingService() throws MalformedURLException{ 
		URL url = new URL("http://www.dummyaddress.com/fbenke");
		assertTrue("Failure while adding supported service for user",UserManager.getUser("fbenke").addService("LinkedIn",url));
		assertTrue("Failure while removing service for user",UserManager.getUser("fbenke").removeService("LinkedIn"));
		assertFalse("Failure while removing non-existent service from user",UserManager.getUser("fbenke").removeService("LinkedIn"));
		assertTrue("Wrong list size after removing services",UserManager.getUser("fbenke").getServiceList().size()==2);
	}

	@Test
	public void testAddingUnsupportedService() throws MalformedURLException{ 
		URL url = new URL("http://www.dummyaddress.com/fbenke");
		assertFalse("Failure while adding unsupported service for user",UserManager.getUser("fbenke").addService("Vimeo", url));
	}
	
	@Test
	public void testAddingDuplicateService() throws MalformedURLException{ 
		URL url = new URL("http://www.dummyaddress.com/fbenke");
		
		/*for(String s: UserManager.getUser("fbenke").getServiceList().keySet()){
			System.out.println(s);
		}*/

		assertFalse("Failure while adding duplicate service for user",UserManager.getUser("fbenke").addService("Twitter", url));
	}

	@Test
	public void testServiceListSize(){ 
		assertTrue("Wrong list size for supported services",UserManager.getUser("fbenke").getServiceList().size()==2);
	}

	@Test
	public void testNumberOfFollowers(){ 
		assertTrue("Failure counting the number of followers",UserManager.getUser("fbenke").getFollowingList().size() == 5);
	}

	@Test
	public void testRemoveFollowing(){ 
		assertTrue("Failure while removing user from following list", UserManager.getUser("fbenke").removeFollowing("bsholes"));
		assertTrue("Failure while restoring following list", UserManager.getUser("fbenke").addFollowing("bsholes"));
		assertTrue("Failure while restoring following list", (UserManager.getFollowers(UserManager.getUser("fbenke").getUsername())).size()==4);
	}

	@Test
	public void testRemoveFollowingNegative(){ 
		assertFalse("Failure while removing user from following list (negative)", UserManager.getUser("fbenke").removeFollowing("ccasey"));
	}

	@Test
	public void testUserListSize(){
		assertTrue("Wrong list size for user list",UserManager.getUserList().size() == 7);
	}

	@Test
	public void testRemoveUserFailure(){
		assertFalse("Failure while removing user from user list (negative)", UserManager.removeUser("rtanksley"));
	}

	@Test
	public void testRemoveUser(){
		assertTrue("Failure while removing user from user list", UserManager.removeUser("ccasey"));
		User colin = new User("Colin", "Casey", "ccasey", "colin@meltwater.org", "financial modeling", 
			"Financial Specialist at MEST");
		assertTrue("Failure while restoring user list", UserManager.addUser(colin));
	}

	@Test
	public void testRemoveUniversity() throws MalformedURLException{
		URL url = new URL("http://www.dummyUR.com/");
		GregorianCalendar startDate = new GregorianCalendar(2006,9,1); 	
		GregorianCalendar endDate = new GregorianCalendar(2009,8,30); 	
		University test = new University("Test University", "Bachelor of Science",
			"Business Information Systems", startDate, endDate, url, "Berlin", url);
		assertTrue("Failure while adding university attendance",  UserManager.getUser("fbenke").addUniversity(test));
		assertTrue("Failure while removing university attendance", UserManager.getUser("fbenke").removeUniversity(UserManager.getUser("fbenke").getUniversityList().get(2).getUid()));
		assertTrue("Wrong size of university list", UserManager.getUser("fbenke").getUniversityList().size()==2);
	}

	@Test
	public void testRemoveWork() throws MalformedURLException{
		URL url = new URL("http://www.dummyUR.com/");
		GregorianCalendar startDate = new GregorianCalendar(2020,9,1); 	
		GregorianCalendar endDate = new GregorianCalendar(2021,8,30);
		Work test = new Work("Test Work", "BA Student", "6 internships in various departments",
		startDate, endDate, url, "Berlin", url);
		assertTrue("Failure while adding work", UserManager.getUser("fbenke").addWork(test));
		assertTrue("Failure while removing work",UserManager.getUser("fbenke").removeWork(UserManager.getUser("fbenke").getWorkList().get(4).getUid()));
		assertTrue("Wrong size of work list", UserManager.getUser("fbenke").getWorkList().size()==4);
	}


	@Test
	public void testSortingUniversities(){
		/*for(University u: UserManager.getUser("fbenke").getUniversityList()){
			System.out.println(u.getName()+", "+u.getUid());
		}*/

		assertTrue("Universities are not in order", 
			UserManager.getUser("fbenke").getUniversityList().get(0).getName().equals("Berlin School of Economics and Law") && 
			UserManager.getUser("fbenke").getUniversityList().get(1).getName().equals("Hasso Plattner Institute"));
	}

	@Test
	public void testSortingWork(){
		/*for(Work w: UserManager.getUser("fbenke").getWorkList()){
			System.out.println(w.getName()+","+w.getUid());
		}*/

		assertTrue("Work list is not in order", 
			UserManager.getUser("fbenke").getWorkList().get(0).getName().equals("IBM") &&
			UserManager.getUser("fbenke").getWorkList().get(1).getName().equals("University of Potsdam") && 
			UserManager.getUser("fbenke").getWorkList().get(2).getName().equals("Upside Shopping GmbH") &&
			UserManager.getUser("fbenke").getWorkList().get(3).getName().equals("Meltwater Entrepreneurial School of Technology"));
	}

	@Test
	public void testUniqueCareerStages(){
		assertTrue("Ids for career stages are not unique", findDuplicates(UserManager.getUser("fbenke").getUniversityList()) && 
			findDuplicates(UserManager.getUser("fbenke").getWorkList()));	
	}

	private static boolean findDuplicates(ArrayList<? extends CareerStage> list){
		HashSet<Long> uniqueList = new HashSet<Long>();
		int i = 0;
		for(CareerStage obj : list) {
			uniqueList.add(obj.getUid());
			i++;
		}

		return uniqueList.size() == list.size();
	}
}