package mest.entity;

import java.util.GregorianCalendar;
import java.util.Calendar;
import java.net.URL;

/** class describing generic career stages */
public abstract class CareerStage{
    protected long uid;
    protected String name;
    protected GregorianCalendar startDate;
    protected GregorianCalendar endDate;
    protected URL website;
    protected String location;
    protected URL locationURL;
    private static long numberOfDevelopmentIDs = 0;


    /**
    This constructor assigns the passed values to the newly created object and creates a new uid,
    that is unique among all career stage objects by using numbers starting from 0..n.
    @param name short description of the career stage
    @param startDate beginning of the stage
    @param endDate end of the stage
    @param website url of the institution's website 
    @param location of the institution
    @param locationURL url pointing to the map location of the institution
    */
    public CareerStage(String name, GregorianCalendar startDate, GregorianCalendar endDate, 
        URL website, String location, URL locationURL){
        this.name = name;
        this.startDate = startDate;
        this.endDate = endDate;
        this.website = website;
        this.locationURL = locationURL;
        this.location= location;
        this.uid = numberOfDevelopmentIDs;
        numberOfDevelopmentIDs++;
    }

    public String getName() {
        return this.name;
    }

    public void setName(String name) {
        this.name = name;
    }

    public GregorianCalendar getStartDate() {
        return startDate;
    }

    public void setStartDate(GregorianCalendar startDate) {
        this.startDate = startDate;
    }

    public GregorianCalendar getEndDate() {
        return endDate;
    }

    public void setEndDate(GregorianCalendar endDate) {
        this.endDate = endDate;
    }

    public URL getWebsite() {
        return this.website;
    }

    public void setWebsite(URL website) {
        this.website = website;
    }

    public String getLocation() {
        return this.location;
    }

    public void setLocation(String location) {
        this.location = location;
    }

    public URL getLocationURL() {
        return this.locationURL;
    }

    public void setLocationURL(URL locationURL) {
        this.locationURL = locationURL;
    }

    public long getUid() {
        return this.uid;
    }

    public void setUid(long uid) {
        this.uid = uid;
    }

}
