package mest.entity;
import java.util.ArrayList;


/** class for managing all the user instances of the application in a private list called userList*/
public class UserManager{
	
    /**list of supported services*/
	private static ArrayList<User> userList = new ArrayList<User>();


    public static ArrayList<User> getUserList(){
        return userList;
    }

    /**
    Adds a user to the list of users.
    @param user to be added
    @return boolean describing if addding was successful
    */
	public static boolean addUser(User user){
        for(User existingUser : userList){
            if(existingUser.getUsername().equals(user.getUsername()))
                return false;
        }
        return userList.add(user);
    }

    /**
    Removes a user from the list of users.
    @param username of the user to be removed
    @return boolean describing if removing was successful
    */
    public static boolean removeUser(String username){
        for (User u : userList){
            if(u.getUsername().equals(username)){
                return userList.remove(u);
            }
        }
        return false;
    }

    /**
    Convenience method. Returns the user object from the list of users for the given username.
    @param username of the user to be retrieved
    @return corresponding user object
    */
    public static User getUser(String username){
        for (User u : userList){
            if(u.getUsername().equals(username)){
                return u;
            }
        }
        return null;
    }

    /**
    Gets a list of followers for a given username.
    @param username of the considered user
    @return list of usernames of followers for the considered user
    */
    public static ArrayList<String> getFollowers(String username){
    	ArrayList<String> followers = new ArrayList<String>();
    	for(User pickedUser : userList){
    		if(pickedUser.getFollowingList().contains(username)){
    			followers.add(pickedUser.getUsername());
    		}
    	}
    	return followers;
    }
}