package mest.utility;
import java.util.Comparator;
import mest.entity.CareerStage;
import java.util.Calendar;

/** class describing the natural order of career stages */
public class CareerStageComparator implements Comparator<CareerStage>{

	/**
	Compare career stages by end date.
	If the month and year of the end date are identical,
	the time span of employment/enrolment is used to decide the order.
	*/
	public int compare(CareerStage career1, CareerStage career2){
		if(career1.getEndDate().get(Calendar.DAY_OF_MONTH) == 
			career2.getEndDate().get(Calendar.DAY_OF_MONTH) && 
			career1.getEndDate().get(Calendar.YEAR) == 
			career2.getEndDate().get(Calendar.YEAR)){
			long timeSpanCareer1 = career1.getEndDate().getTimeInMillis() - 
				career1.getStartDate().getTimeInMillis();
			long timeSpanCareer2 = career2.getEndDate().getTimeInMillis() - 
				career2.getStartDate().getTimeInMillis();
			return compareLong(timeSpanCareer1, timeSpanCareer2);
		}else{
			return career1.getEndDate().compareTo(career2.getEndDate());
		}
	}

	private int compareLong(long l1, long l2){
		if(l1 > l2){
			return 1;
		}else if(l2 > l1){
			return -1;
		}else{
			return 0;
		}
	}
	
}